@@ -1,5 +1,5 @@
 # Original work Copyright (c) 2016 Microsoft Corporation. All rights reserved.
-# Modified work Copyright (c) 2020 Fabio Sigrist. All rights reserved.
+# Modified work Copyright (c) 2020 - 2024 Fabio Sigrist. All rights reserved.
 # Licensed under the Apache License Version 2.0 See LICENSE file in the project root for license information.
 
 #' @importFrom R6 R6Class
@@ -84,10 +84,10 @@ gpb.cv <- function(params = list()
                    , data
                    , nrounds = 100L
                    , gp_model = NULL
+                   , line_search_step_length = FALSE
                    , use_gp_model_for_validation = TRUE
                    , fit_GP_cov_pars_OOS = FALSE
                    , train_gp_model_cov_pars = TRUE
-                   , reuse_learning_rates_gp_model = FALSE
                    , folds = NULL
                    , nfold = 4L
                    , label = NULL
@@ -136,7 +136,7 @@ gpb.cv <- function(params = list()
   
   params$use_gp_model_for_validation <- use_gp_model_for_validation
   params$train_gp_model_cov_pars <- train_gp_model_cov_pars
-  params$reuse_learning_rates_gp_model <- reuse_learning_rates_gp_model
+  params$line_search_step_length <- line_search_step_length
   
   # set some parameters, resolving the way they were passed in with other parameters
   # in `params`.
@@ -981,9 +981,9 @@ gpb.grid.search.tune.parameters <- function(param_grid
                                             , num_try_random = NULL
                                             , nrounds = 100L
                                             , gp_model = NULL
+                                            , line_search_step_length = FALSE
                                             , use_gp_model_for_validation = TRUE
                                             , train_gp_model_cov_pars = TRUE
-                                            , reuse_learning_rates_gp_model = FALSE
                                             , folds = NULL
                                             , nfold = 4L
                                             , label = NULL
@@ -1075,9 +1075,9 @@ gpb.grid.search.tune.parameters <- function(param_grid
                     , data = data
                     , nrounds = nrounds
                     , gp_model = gp_model
+                    , line_search_step_length = line_search_step_length
                     , use_gp_model_for_validation = use_gp_model_for_validation
                     , train_gp_model_cov_pars = train_gp_model_cov_pars
-                    , reuse_learning_rates_gp_model = reuse_learning_rates_gp_model
                     , folds = folds
                     , nfold = nfold
                     , label = label
